#include <Arduino.h>

#define USE_WIFI	1   // for TerminalDevice
#define USE_XBEE	0   // for RelayDevice
#define USE_GPS		1

#if USE_WIFI
#include "ESP8266.h"
#endif // #if USE_WIFI

#if USE_XBEE
#include "stdtypes.h"
#include "RingBuffer.h"
#include "XBeeFrame.h"
#include "XBeeAPIRxFrame.h"
#include "XBee.h"
#include "HanaXbee.h"
using namespace std;
#endif // #if USE_XBEE

#if USE_GPS
#include "gps.h"
#endif // #if USE_GPS

/****************************************************************************/
/* 共通の定義                                                               */
/****************************************************************************/
#define LOOP_DELAY      10              // ループ毎のディレイタイム(ms)
#define USB_Serial      Serial          // USBシリアル通信ポート
#define USB_Baud        9600	        // USBシリアル通信ボーレート

/****************************************************************************/
/* Wi-Fi関連の定義                                                          */
/****************************************************************************/
#if USE_WIFI
#define WLAN_SSID       "XXXXXXXXXX"					// AP Name
#define WLAN_PASS       "XXXXXXXXXX"					// AP PassWord
#define HOST_NAME       "104.199.222.XXX"				// HOST Address
#define HOST_URL		"/r-king/hana/webapi.php?"
#define HOST_PORT       (80)							// HOST Port
#define ESP_Serial      Serial2							// WiFi Module Port
#define ESP_Baud        115200							// WiFi Module Port
ESP8266 wifi;
#endif // #if USE_WIFI

/****************************************************************************/
/* Xbee関連の定義                                                           */
/****************************************************************************/
#if USE_XBEE
#define XBEE_SERIAL		Serial2
#define XBEE_BAUD		9600
u1 coodinator_addr64[] = { 0x00,0x13,0xA2,0x00, 0x40,0xF7,0x3B,0xE6 };
u1 coodinator_addr16[] = { 0xFF, 0xFE };
HanaXBee xbee;
#endif // #if USE_XBEE

/****************************************************************************/
/* GPS関連の定義                                                            */
/****************************************************************************/
#if USE_GPS
#define GPS_Baud		9600
#define GPS_TX			8
#define	GPS_RX			7
double					g_lat,g_lon,g_alt;
int                     g_year,g_month,g_day,g_hour,g_min,g_sec;
bool                    fixGPS = false;
SerialGPS 				gps(GPS_TX, GPS_RX, GPS_Baud);
SerialGPS::GpsCallbackTable cbfuncs;
#endif // #if USE_GPSS

/****************************************************************************/
/* 人検知関連の定義     		                                             */
/****************************************************************************/
#define IR_DETECT_INT		0	// 人体検知割込み番号
#define IR_DETECT_PIN		2	// 人体検知割込みピン
#define SW_DETECT_INT		1	// SW割込み番号
#define SW_DETECT_PIN		3	// SW割込みピン

/****************************************************************************/
/* LED関連の定義                                                            */
/****************************************************************************/
#define LED_R 22 // Red LED
#define LED_G 23 // Green LED
#define LED_B 24 // Blue LED

/****************************************************************************/
/* グローバル変数の定義                                                     */
/****************************************************************************/
int sw_on = 0;
int ir_on = 0;
bool enableGPS	= false;
bool enableWiFi = false;
bool enableXbee = false;

/****************************************************************************/
/* プロトタイプ宣言                                                         */
/****************************************************************************/
#if USE_WIFI
static int setupWiFi(void);
static int sendWiFi(void);
static void resetWiFi(void);
#endif // #if USE_WIFI

#if USE_XBEE
static int setupXbee(void);
static int sendXbee(void);
#endif // #if USE_XBEE

#if USE_GPS
static int setupGPS(void);
static void gps_log_callback(uint8_t *data);
static void gps_gga_callback(SerialGPS::NMEA_GGA *nmea_gga);
static void gps_zda_callback(SerialGPS::NMEA_ZDA *nmea_zda);
static void gpsUpdateLocation(double lat, double lon, double altitude);
#endif // #if USE_GPS

static void detect_IRSensor();		// 赤外線センサによる検出
static void detect_SW();			// スイッチによる検出

/****************************************************************************/
//	機能		: 起動時のセットアップ処理
//	引数		: なし
//	返り値		: なし
//	備考		: 起動時一番最初に呼ばれる
/****************************************************************************/
void setup()
{
    USB_Serial.begin(USB_Baud);
    USB_Serial.print("Setup START\r\n");
    
    // 動作確認用LED
    pinMode(LED_R, OUTPUT);
    pinMode(LED_G, OUTPUT);
    pinMode(LED_B, OUTPUT);
    digitalWrite(LED_R, LOW);
    digitalWrite(LED_G, HIGH);
    digitalWrite(LED_B, HIGH);

	// setup IR Sensor
	pinMode(IR_DETECT_PIN, INPUT_PULLUP);							// 入力に設定
    digitalWrite(IR_DETECT_PIN, LOW);								// LOWに設定
    attachInterrupt(IR_DETECT_INT, detect_IRSensor, RISING);		// 割込み登録(立ち上がり)
    
	// setup SW
	pinMode(SW_DETECT_PIN, INPUT_PULLUP);							// 入力に設定
	digitalWrite(SW_DETECT_PIN, HIGH);								// HIGHに設定
	attachInterrupt(SW_DETECT_INT, detect_SW, FALLING);				// 割込み登録(立ち下がり)

#if USE_GPS
	enableGPS = setupGPS();
#endif // #if USE_GPS

#if USE_WIFI
	enableWiFi = setupWiFi();
#endif // #if USE_WIFI

#if USE_XBEE
	enableXbee = setupXbee();
#endif // #if USE_XBEE

	USB_Serial.print("Setup DONE\r\n");
}

/****************************************************************************/
//	機能		: メインループ処理
//	引数		: なし
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void loop()
{
	
#if USE_XBEE
	xbee.func_hi();
	xbee.func_mid();
#endif // #if USE_XBEE

#if USE_GPS
    if(fixGPS == false)
    {
        gps.Processing();
        gps.RcvIsrHandler();
    }
#endif // #if USE_GPS

	if((ir_on == true) || (sw_on == true))
	{
#if USE_WIFI
		if((enableWiFi == true))
		{
			sendWiFi();
		}
#endif // #if USE_WIFI
#if USE_XBEE
		if(enableXbee == true)
		{
			sendXbee();
		}
#endif // #if USE_XBEE
	}

    delay(LOOP_DELAY);
}

/****************************************************************************/
//	機能		: IR検知処理
//	引数		: なし
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void detect_IRSensor()
{
	ir_on = true;
}

/****************************************************************************/
//	機能		: SW検知処理
//	引数		: なし
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void detect_SW()
{
	sw_on = true;
}

#if USE_WIFI
/****************************************************************************/
//	機能		: Wi-Fi通信機能のセットアップ
//	引数		: なし
//	返り値		: ret           FALSE(0):失敗 TRUE(1):成功
//	備考		: なし
/****************************************************************************/
int setupWiFi()
{
    int ret    = false;
    
    // FWバージョンを取得する
    wifi.begin(ESP_Serial, ESP_Baud);
    
    // Wi-Fiモジュールをリセットする
    resetWiFi();
    
    USB_Serial.print("FW Version:");

    USB_Serial.println(wifi.getVersion().c_str());
    
    // 接続の準備
    if (wifi.setOprToStationSoftAP()) {
        USB_Serial.print("to station + softap ok\r\n");
    } else {
        USB_Serial.print("to station + softap err\r\n");
        //return ret;
    }

    // APに接続する
    if (wifi.joinAP(WLAN_SSID, WLAN_PASS)) {
        USB_Serial.print("Join AP success\r\n");
        USB_Serial.print("IP:");
        USB_Serial.println( wifi.getLocalIP().c_str());       
    } else {
        USB_Serial.print("Join AP failure\r\n");
        return ret;
    }

    // コネクションを1つのみに設定
    if (wifi.disableMUX()) {
        USB_Serial.print("single ok\r\n");
    } else {
        USB_Serial.print("single err\r\n");
        return ret;
    }
    
    USB_Serial.print("setup end\r\n");
    ret = true;

    return ret;
}

/****************************************************************************/
//	機能		: Wi-Fi送信機能
//	引数		: なし
//	返り値		: ret           FALSE(0):失敗 TRUE(1):成功
//	備考		: なし
/****************************************************************************/
int sendWiFi()
{
    USB_Serial.print("send\r\n");
    
    int ret = false;
    bool sendRet = false;

    if (wifi.createTCP(HOST_NAME, HOST_PORT)) {
        USB_Serial.print("create tcp ok\r\n");
    } else {
        USB_Serial.print("create tcp err\r\n");
        return ret;
    }
    
	char send_buf[1024] = {'\0'};
	
	sprintf(send_buf, "GET %saction=add&token=001&sw_on=%d&ir_on=%d&sound_on=0&lat=%lf&lon=%lf&height=%lf HTTP/1.1\r\nHost: %s\r\n\r\n",
	    HOST_URL,
	    sw_on,
	    ir_on,
	    g_lat,
	    g_lon,
	    g_alt,
	    HOST_NAME);

	USB_Serial.print(send_buf);
    sendRet = wifi.send((const uint8_t*)send_buf, strlen(send_buf));
    if(sendRet == true) {
    	USB_Serial.print("Success!!\r\n");
    } else {
    	USB_Serial.print("Fail!!\r\n");
    	return ret;
    }
    
    uint8_t rcv_buf[2048] = {0};
    uint32_t len = wifi.recv(rcv_buf, sizeof(rcv_buf), 10000);
    if (len > 0) {
        USB_Serial.print("Received:[");
        for(uint32_t i = 0; i < len; i++) {
            USB_Serial.print((char)rcv_buf[i]);
        }
        USB_Serial.print("]\r\n");
    }
    
    if (wifi.releaseTCP()) {
        USB_Serial.print("release tcp ok\r\n");
    } else {
        USB_Serial.print("release tcp err\r\n");
    }
    ret = true;
    sw_on = 0;
    ir_on = 0;
    
    return ret;
}

/****************************************************************************/
//	機能		: Wi-Fiリセット機能
//	引数		: なし
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void resetWiFi()
{
    wifi.kick();
    delay(100);
    wifi.restart();
    delay(100);
}
#endif // #if USE_WIFI

#if USE_XBEE
/****************************************************************************/
//	機能		: Xbee機能のセットアップ
//	引数		: なし
//	返り値		: ret           FALSE(0):失敗 TRUE(1):成功
//	備考		: なし
/****************************************************************************/
int setupXbee()
{
	int ret = true;
	XBEE_SERIAL.begin(XBEE_BAUD);
	xbee.init(&Serial1,coodinator_addr64,coodinator_addr16);
	return ret;
}

/****************************************************************************/
//	機能		: Xbee送信機能
//	引数		: なし
//	返り値		: ret           FALSE(0):失敗 TRUE(1):成功
//	備考		: なし
/****************************************************************************/
int sendXbee()
{
    int ret     = false;
	int det_ret = false;
	int loc_ret = false;

    det_ret = xbee.TxDetectInfo( sw_on, ir_on, 0, g_hour, g_min, g_sec);					// Detect Info フレーム送信処理
    delay(1);
	loc_ret = xbee.TxLocationInfo( 0, g_lat, 0, g_lon, g_hour, g_min, g_sec);	            // Location Info フレーム送信処理
    
    if((det_ret == true) && (loc_ret == true))
    {
        ret = true;
    }
	return ret;
}
#endif // #if USE_XBEE

#if USE_GPS
/****************************************************************************/
//	機能		: GPS機能のセットアップ
//	引数		: なし
//	返り値		: ret           FALSE(0):失敗 TRUE(1):成功
//	備考		: なし
/****************************************************************************/
int setupGPS()
{
	cbfuncs.log_cbfunc = NULL;
	cbfuncs.gga_cbfunc = gps_gga_callback;
	cbfuncs.gll_cbfunc = NULL;
	cbfuncs.gsa_cbfunc = NULL;
	cbfuncs.gsv_cbfunc = NULL;
	cbfuncs.rmc_cbfunc = NULL;
	cbfuncs.vtg_cbfunc = NULL;
	cbfuncs.zda_cbfunc = NULL;
	
	gps.SetGpsCallback(&cbfuncs);
	gps.RcvIsrEnable();

	return true;
}

/****************************************************************************/
//	機能		: GPSログコールバック機能
//	引数		: data      	受信データ
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void gps_log_callback(uint8_t *data)
{
//	printf("%s\r\n", data);
}

/****************************************************************************/
//	機能		: GPS GGAコールバック機能
//	引数		: nmea_gga     	NMEA GGAデータ
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void gps_gga_callback(SerialGPS::NMEA_GGA *nmea_gga)
{
	// 衛星3個以上補足し、位置情報がGIXされていれば
    if( (nmea_gga->num_of_satellites) > 3 && (nmea_gga->fix_quality) > 0)
    {
        fixGPS = true;
        
        gpsUpdateLocation(nmea_gga->latitude, nmea_gga->longitude, nmea_gga->altitude);
        
        USB_Serial.print("LOCATION UPDATE LAT:");
        USB_Serial.print(g_lat);
        USB_Serial.print(", LON:");
        USB_Serial.print(g_lon);
        USB_Serial.print(", ALT:");
        USB_Serial.print(g_alt);
        USB_Serial.print("\r\n");
    }

}

/****************************************************************************/
//	機能		: 60進法から10進法への変換
//	引数		: lat,lon     	60進法データ
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void gpsUpdateLocation(double lat, double lon, double altitude)
{
	double d_lat,d_lon,m_lat,m_lon;
	
	// lat
	d_lat = int( lat / 100 );
	m_lat = ( lat - d_lat * 100 ) / 60;
	g_lat = d_lat + m_lat;
	
	// lon
	d_lon = int( lon / 100 );
	m_lon = (lon - d_lon * 100 ) / 60;
	g_lon = d_lon + m_lon;
	
	// alt
	g_alt = altitude;
}

/****************************************************************************/
//	機能		: GPS ZDAコールバック機能
//	引数		: nmea_ZDA     	NMEA ZDAデータ
//	返り値		: なし
//	備考		: なし
/****************************************************************************/
void gps_zda_callback(SerialGPS::NMEA_ZDA *nmea_zda)
{
    g_year      = nmea_zda->year;
    g_month     = nmea_zda->month;
    g_day       = nmea_zda->day;
    g_hour      = (nmea_zda->hour) + (nmea_zda->hour_diff);
    g_min       = (nmea_zda->min)  + (nmea_zda->min_diff);
    g_sec       = nmea_zda->sec;
    
    USB_Serial.print("TIME UPDATE ");
    USB_Serial.print(g_year);
    USB_Serial.print("/");
    USB_Serial.print(g_month);
    USB_Serial.print("/");
    USB_Serial.print(g_day);
    USB_Serial.print(" ");
    USB_Serial.print(g_hour);
    USB_Serial.print(":");
    USB_Serial.print(g_min);
    USB_Serial.print(":");
    USB_Serial.print(g_sec);
    USB_Serial.print("\r\n");
}
#endif // #if USE_GPS
