#include <string.h>
#include "queue.h"

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

QueueData::QueueData()
{
    Init();
}

QueueData::~QueueData()
{
    // Nothing.
}

void QueueData::Init(void)
{
    SetStatus(QDATA_EMPTY);
}

void QueueData::SetStatus(QueueData::QDataState state)
{
    if(state == QDATA_EMPTY){
        msg[0] = '\0';
        len = 0;
    }
    this->state = state;
}

QueueData::QDataState QueueData::GetStatus(void)
{
    return this->state;
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

Queue::Queue()
{
    Init();
}

Queue::~Queue()
{
}

void Queue::Init(void)
{
    int32_t ptr;
    
    wque = 0;
    rque = 0;
    for(ptr = 0; ptr <= MAX_QUEUE_SIZE; ptr++){
        queue[ptr].Init();
    }
}

bool Queue::Enqueue(QueueData *src)
{
    if(queue[wque].GetStatus() != QueueData::QDATA_EMPTY){
        // キュー詰まり状態
        return false;
    }
    
    if(src->len > QueueData::QUEUE_BUFFER_SIZE){
        // キューサイズより格納したいデータが大きい
        return false;
    }
    
    // キュー格納
    memcpy(queue[wque].msg, src->msg, src->len);
    queue[wque].len = src->len;
    
    // キュー格納完了
    queue[wque].SetStatus(QueueData::QDATA_READY);
    
    // ライトポインタ循環
    wque++;
    if(wque >= MAX_QUEUE_SIZE){
        wque = 0;
    }
    
    return true;
}

bool Queue::Dequeue(QueueData **dst)
{
    *dst = NULL;
    if(queue[rque].GetStatus() != QueueData::QDATA_READY){
        // キュー無し
        return false;
    }
    
    // キュー取り出し
    *dst = &queue[rque];

    return true;
}

bool Queue::DequeueComplete(void)
{
    if(queue[rque].GetStatus() != QueueData::QDATA_READY){
        // キュー無し
        return false;
    }

    // キュー初期化
    queue[rque].SetStatus(QueueData::QDATA_EMPTY);
    
    // リードポインタ循環
    rque++;
    if(rque >= MAX_QUEUE_SIZE){
        rque = 0;
    }
    
    return true;
}
