#ifndef _MESSAGE_QUEUE_
#define _MESSAGE_QUEUE_

#include "peer.h"
#ifdef USE_MBED
#include "mbed.h"
#else  // USE_MBED
#include <Arduino.h>
#endif // USE_MBED

/**
 * Queue data buffer
 */
struct QueueData {
public:
    // Queue data status
    enum QDataState {
        QDATA_EMPTY = 0,
        QDATA_READY = 1,
    };
    
    // Queue data buffer size
    enum QDataSize {
        QUEUE_BUFFER_SIZE = 128,
    };

    // Queue data buffer
    uint8_t msg[QUEUE_BUFFER_SIZE];
    int32_t len;

    /**
     * Constructor
     */
    QueueData();

    /**
     * Destructor
     */
    ~QueueData();

    /**
     * QueueData message initializing
     */
    void Init(void);
    
    /**
     * Setting a queue status
     */
    void SetStatus(QDataState state);

    /**
     * Getting a queue status
     */
    QDataState GetStatus(void);
    
private:
    QDataState state;           // Queue data state
};

/**
 * Message queue class
 */
class Queue {
public:
    /**
     * Constructor
     */
    Queue();

    /**
     * Destructor
     */
    ~Queue();
    
    /**
     * Init
     */
    void Init(void);
    
    /**
     * Enqueue
     */
    bool Enqueue(QueueData *queue);
    
    /**
     * Dequeue (1/2-pass)
     */
    bool Dequeue(QueueData **queue);

    /**
     * DequeueComplete (2/2-pass)
     */
    bool DequeueComplete(void);
    
private:
    // Message queue buffer size
    #define MAX_QUEUE_SIZE (12)
    // Message queue buffer
    QueueData queue[MAX_QUEUE_SIZE];
    int32_t wque; // Write queue pointer
    int32_t rque; // Read  queue pointer
};
#endif