#include "stdtypes.h"

#ifdef DEBUG
#include "StubSerial.h"
#else
#include "HardwareSerial.h"
#endif

#include "RingBuffer.h"
#include "XBeeFrame.h"
#include "XBeeAPIRxFrame.h"
#include "XBee.h"
#include "HanaXBeeFrame.h"
#include "HanaXBee.h"
#include <stdio.h>

HanaXBee::HanaXBee(void)
{
	m_debuglen = 0;
}

HanaXBee::~HanaXBee(void)
{
}

// 引数		: pserial	制御対象のシリアル
//			: des64		送信先64bitアドレス
//			: des16		送信先16bitアドレス
void HanaXBee::init(HardwareSerial *pserial, const u1 *des64, const u1 *des16 )
{
    XBee::init( pserial );
	m_DestAddr64[0] = des64[0];		// 64bitアドレス登録
	m_DestAddr64[1] = des64[1];
	m_DestAddr64[2] = des64[2];
	m_DestAddr64[3] = des64[3];
	m_DestAddr64[4] = des64[4];
	m_DestAddr64[5] = des64[5];
	m_DestAddr64[6] = des64[6];
	m_DestAddr64[7] = des64[7];
	m_DestAddr64[8] = des64[8];

	m_DestAddr16[0] = des16[0];		// 16bitアドレス登録
	m_DestAddr16[1] = des16[1];
}

// 引数		: str	送信する文字列
// 			: len	文字列長
// 戻り値	: 成功(TRUE)/失敗(FALSE)
u1 HanaXBee::TxString( const s1 *str, u1 len )
{
	HanaTxFrame frame;
	u1	cnt;
	
	SetupTransmitRequest(&frame.tq, m_DestAddr64, m_DestAddr16 );		// Transmit Request 部分作成

	for( cnt=0; cnt<len; cnt++)											// 転送文字列設定
	{
		frame.data.str[cnt] = str[cnt];
	}
	
	return TxTransmitRequest( (TransmitRequest *)&frame, sizeof(frame.tq) + len);
}

// 引数		: sw	スイッチの押下(1:押下検知あり/0:押下検知なし)
//			: human	人検知センサ(1:検知/0:未検知)
//			: mic	マイク検知(A:あ,I:い,U:う,E:え,O:お)
//			: hh	時間
//			: mm	分
//			: ss	秒
u1 HanaXBee::TxDetectInfo( u1 sw, u1 human, u1 mic, u1 hh, u1 mm, u1 ss)
{
	HanaTxFrame frame;

	SetupTransmitRequest(&frame.tq, m_DestAddr64, m_DestAddr16 );		// Transmit Request 部分作成
	
	frame.data.di.detect.byte = (u1)0U;									// 検知情報クリア
	frame.data.di.detect.bit.SOS_sw = sw;								// スイッチ格納
	frame.data.di.detect.bit.Human_det = human;							// 人検知格納
	frame.data.di.Mic = mic;											// マイク格納
	frame.data.di.hh = hh;												// 時間格納
	frame.data.di.mm = mm;												// 分格納
	frame.data.di.dd = ss;												// 秒格納
	
	return TxTransmitRequest( (TransmitRequest *)&frame, sizeof(frame.tq) + sizeof(frame.data.di));	// フレーム送信
}

// 引数		: ns	緯度(N:
//			: lat	
//			: ew	
//			: lon	
//			: hh	
//			: mm	
//			: ss	

u1 HanaXBee::TxLocationInfo( u1 ns, double lat, u1 ew, double lon, u1 hh, u1 mm, u1 ss)
{
	HanaTxFrame frame;

	SetupTransmitRequest(&frame.tq, m_DestAddr64, m_DestAddr16 );		// Transmit Request 部分作成
	
	frame.data.li.ns = ns;
    sprintf( (char*) frame.data.li.Latitude, "%10.4lf", lat );
    frame.data.li.ew = ew;
    sprintf( (char*) frame.data.li.Longitude, "%10.4lf", lon );
    frame.data.li.t  = 'T';
    sprintf( (char*) frame.data.li.UTC, "%2d%2d%2d", hh, mm, ss);
	
	return TxTransmitRequest( (TransmitRequest *)&frame, sizeof(frame.tq) + sizeof(frame.data.li));
}


void HanaXBee::RxATCommandResponse( ATCommand *pframe, u1 len)
{
#ifdef DEBUG
    Serial.print("Hana Received AT Command\n;");
#endif
}


void HanaXBee::RxTransmitStatus( TransmitStatus *pframe, u1 len)
{
#ifdef DEBUG
    Serial.print("Hana Received TransmitStatus\n;");
#endif
}

void HanaXBee::RxReceivePacket( ReceivePacket *pframe, u1 len)
{
#ifdef DEBUG
    Serial.print("Hana Received ReceivePacket\n;");
#endif
}

void HanaXBee::SetupTransmitRequest( TransmitRequest *pframe, const u1 * des64, const u1 * des16 )
{
	pframe[0].Frame_type	=	XBee::TRANSMIT_REQUEST;
	pframe[0].Frame_ID		=	HanaXBee::FRAME_ID;
	pframe[0].DestAddr_64_1	=	des64[0];
	pframe[0].DestAddr_64_2	=	des64[1];
	pframe[0].DestAddr_64_3	=	des64[2];
	pframe[0].DestAddr_64_4	=	des64[3];
	pframe[0].DestAddr_64_5	=	des64[4];
	pframe[0].DestAddr_64_6	=	des64[5];
	pframe[0].DestAddr_64_7	=	des64[6];
	pframe[0].DestAddr_64_8	=	des64[7];
	pframe[0].DestAddr_16_h	=	des16[0];
	pframe[0].DestAddr_16_l	=	des16[1];
	pframe[0].BroadcastRadius=	(u1)0x00U;
	pframe[0].Options		=	(u1)0x00U;
}

