#include "stdtypes.h"
#include "RingBuffer.h"


RingBuffer::RingBuffer(void)
{
	init();
}

RingBuffer::~RingBuffer(void)
{
}

void RingBuffer::init( void )
{
	m_size = (u1)0U;
	m_head = (u1)0U;
	m_tail = (u1)0U;
}

// 引数		: data 格納するデータ
//          : len  データ長
// 戻り値	: 成功(TRUE)/失敗(FALSE)
u1 RingBuffer::set( const u1 * data, u1 len )
{
	u1 ret;
	u1 total;
	u1 cur;
	
	total = m_size + len;				// 最終的に格納されるデータ
	ret = FALSE;						// 戻り値(NG)
	
	if( BUFFER_SIZE > total )			// データ格納可能？
	{
		for( cur=0; cur<len; cur++ )	// データ登録処理
		{
			m_buffer[m_tail] = data[cur];
			m_tail++;
			m_size++;
			if( BUFFER_SIZE <= m_tail )	// 終端に達した場合先頭へ
			{
				m_tail = (u1)0U;
			}
			else
			{
			}
		}
		ret = TRUE;						// 戻り値(OK)
	}
	else
	{
	}
	
	return ret;
}

// 引数		: data 格納データ
// 戻り値	: 成功(TRUE)/失敗(FALSE)
u1 RingBuffer::set( u1 data )
{
	u1 ret;
	u1 total;

	total = m_size + 1;					// 最終的に格納されるデータ
	ret = FALSE;						// 戻り値(NG)

	if( BUFFER_SIZE > total )			// データ格納可能？
	{
		m_buffer[m_tail] = data;
		m_tail++;
		m_size++;
		if( BUFFER_SIZE <= m_tail )		// 終端に達した場合先頭へ
		{
			m_tail = (u1)0U;
		}
		else
		{
		}
		ret = TRUE;						// 戻り値(OK)
	}
	else
	{
		ret = FALSE;
	}

	return ret;
}

// 引数		: data データの格納先
//          : len  データ長
// 戻り値	: 成功(TRUE)/失敗(FALSE)
u1 RingBuffer::get( u1 * data, u1 len )
{
	u1 ret;
	u1 read_size;
	u1 cur;
	
	read_size = 0;
	ret = FALSE;
	
	if( m_size < len )					// 取り出しサイズの調整
	{
		read_size = m_size;				// 指定サイズより大きい場合、格納しているデータ長分
	}
	else
	{
		read_size = len;				// 指定サイズより小さい場合
	}
	
	if( 0 < read_size )					// 取り出しデータ有り？
	{
		for( cur = 0; cur < read_size; cur++ )	// データ取り出し処理
		{
			data[cur] = m_buffer[m_head];
			m_head++;
			m_size--;
			if( BUFFER_SIZE <= m_head )			// 終端に達してた場合、先頭へ
			{
				m_head = ((u1)0U);
			}
			else
			{
			}
		}
		ret = read_size;
	}
	else
	{
	}
	
	return ret;
}


// 引数		: データの格納先
// 戻り値	: 成功(TRUE)/失敗(FALSE)
u1 RingBuffer::get( u1 * data )
{
	u1 ret;

	ret = FALSE;

	if( 0 < m_size )					// 取り出しデータ有り？
	{
		data[0] = m_buffer[m_head];
		m_head++;
		m_size--;
		if( BUFFER_SIZE <= m_head )		// 終端に達してた場合、先頭へ
		{
			m_head = ((u1)0U);
		}
		else
		{
		}
		ret = TRUE;
	}
	else
	{
		ret = FALSE;
	}

	return ret;
}
