#include "stdtypes.h"

#ifdef DEBUG
#include "StubSerial.h"
#else
#include "HardwareSerial.h"
#endif

#include "RingBuffer.h"
#include "XBeeFrame.h"
#include "XBeeAPIRxFrame.h"
#include "XBee.h"

XBee::XBee(void )
{
	init();
}

// 引数		: 操作対象のシリアルオブジェクト
XBee::XBee( HardwareSerial * pserial )
{
	init( pserial );
}

XBee::~XBee(void)
{
}

void XBee::init( void )
{
	init( (HardwareSerial *)0 );
}

// 引数		: 操作対象のシリアルオブジェクト
void XBee::init(HardwareSerial * serial)
{
	m_pSerial = serial;
}

// 引数		: pframe 送信対象のフレーム
//			: len    フレームのデータ長
// 戻り値	: 成功(TRUE)/失敗(FALSE)
u1 XBee::transmit( XBeeTxFrame *pframe, u1 len )
{
	u1 ret;
	u1 cnt;
	u4 sum;

	ret = FALSE;								// 戻り値(失敗)
	sum = ((u4)0U);								// チェックサム値クリア

	if((RingBuffer::BUFFER_SIZE - m_TxBuff.size()) > (XBee::TXAPIFRAME_MIN_SIZE + len))	// 送信用リングバッファに格納可能か？
	{
		m_TxBuff.set(XBee::FRAME_DELIMITER);	// フレームデリミタ
		m_TxBuff.set( (u1)((len>>8) & 0xFF) );	// 送信データ長(MSB)
		m_TxBuff.set( (u1)(len&0xff) );			// 送信データ長(LSB)
		for( cnt=0; cnt<len; cnt++)				// データ格納ループ
		{
			sum = sum + (u4)pframe->buff[cnt];	// チェックサム計算
			m_TxBuff.set( pframe->buff[cnt] );	// データ格納
		}
		m_TxBuff.set( (u1)(0x000000ff-sum) );	// チェックサム格納

		ret = TRUE;								// 戻り値(成功)
	}
	else
	{
	}

	return ret;
}

// 受信処理
void XBee::receive( void )
{
	XBeeRxFrame * pframe;
	u1 ret;

	if( 0 < m_RxBuff.size() )				// 受信バッファにデータ有り？
	{
		ret = m_RxFrame.build( &m_RxBuff );	// 受信フレーム構築

		if( TRUE == ret )					// 受信フレーム正常？
		{
			pframe = m_RxFrame.get();		// 受信フレーム取得
#ifdef DEBUG
            Serial.print("Frame type:");
            Serial.print(pframe->com.Frame_type, 16);
            Serial.print("\n");
#endif
			switch( pframe->com.Frame_type )
			{
			case AT_COMMAND_RESPONSE:
				RxATCommandResponse( (ATCommand *)pframe, m_RxFrame.size());
				break;

			case MODEM_STATUS:
				RxModemStatus( (ModemStatus *)pframe, m_RxFrame.size());
				break;

			case TRANSMIT_STATUS:
				RxTransmitStatus( (TransmitStatus *)pframe, m_RxFrame.size());
				break;

			case RECEIVE_PACKET:
				RxReceivePacket( (ReceivePacket *)pframe, m_RxFrame.size());
				break;

			case EXPLICIT_RX_INDICATOR:
				RxExplicitRxIndicator( (ExplicitRxIndicator *)pframe, m_RxFrame.size());
				break;

			case IO_DATA_SAMPLE_RX_INDICATOR:
				RxIODataSampleRxIndicator( (DataSampleRxIndicator *)pframe, m_RxFrame.size());
				break;

			case XBEE_SENSOR_READ_INDICATOR:
				RxXBeeSensorReadIndicator( (XBeeSensorReadIndicator *)pframe, m_RxFrame.size());
				break;

			case NODE_IDENTIFICATION_INDICATOR:
				RxNodeIdentificationIndicator( (NodeIdentificationIndicator *)pframe, m_RxFrame.size());
				break;

			case REMOTE_COMMAND_RESPONSE:
				RxRemoteCommandResponse( (RemoteCommandResponse *)pframe, m_RxFrame.size());
				break;

			case EXTENDED_MODEM_STATUS:
				RxExtendedModemStatus( (ExtendedModemStatus *)pframe, m_RxFrame.size());
				break;

			case OVER_THE_AIR_FIRMWARE_UPDATE_STATUS:
				RxOverTheAirFirmwareUpdateStatus( (OverTheAirFirmwareUpdateStatus *)pframe, m_RxFrame.size());
				break;

			case ROUTE_RECORD_INDICATOR:
				RxRouteRecordIndicator( (RouteRecordIndicator *)pframe, m_RxFrame.size());
				break;

			case MANY_TO_ONE_ROUTE_REQUEST_INDICATOR:
				RxManyToOneRouteRequestIndicator( (ManyToOneRouteRequestIndicator *)pframe, m_RxFrame.size());
				break;

			default:
				break;
			}

			m_RxFrame.init();				// 受信フレームクリア
		}
		else
		{
											// 受信フレームは受信途中
		}
	}
	else
	{
	}
	return;
}

void XBee::func_hi( void )
{
	int len;
	int data;
	u1 ulen;
	u1 cnt;

	if( ((HardwareSerial*)0) != m_pSerial )
	{
		len = m_pSerial->available();			// 受信データ確認
		if( 0 < len )							// 受信データ有り？
		{
#ifdef DEBUG
			Serial.print("Rx:");
#endif
			for( cnt=0; cnt<(u1)len; cnt++ )	// 受信データを受信リングバッファに格納
			{
				data = m_pSerial->read();		// 受信データ読み出し
				if( 0 <= data)					// 受信データあり？
				{
					m_RxBuff.set( (u1)data );	// データを受信リングバッファに格納
#ifdef DEBUG
                    Serial.print( data, 16);
                    Serial.print( " " );
#endif
				}
				else
				{
				}
			}
#ifdef DEBUG
			Serial.print("\n");
#endif
		}
		else
		{
		}

		ulen = m_TxBuff.size();					// 送信リングバッファにデータ有り？
		if( 0 < ulen )
		{
			ulen = m_TxBuff.get( m_TxTemp, XBee::TXTEMP_SIZE);	// 送信リングバッファから一定サイズのデータを取り出し
			m_pSerial->write( m_TxTemp, ulen);	// シリアルにデータを転送
			
#ifdef DEBUG
			Serial.print("Tx:");
			for( cnt=0; cnt<ulen; cnt++)
			{
			    Serial.print( m_TxTemp[cnt], 16);
			    Serial.print( " " );
			}
			Serial.print( "\n" );
#endif
		}
		else
		{
		}
	}
	else
	{
	}
}

void XBee::func_mid( void )
{
	receive();		// 受信処理
}

