#ifndef __XBEE_H__
#define __XBEE_H__

// XBeeクラス
class XBee
{
public:
	enum
	{
		TXTEMP_SIZE 								= 0x40,		// シリアルにデータを渡すためのバッファサイズ(64byte)
		TXAPIFRAME_MIN_SIZE 						= 0x04,		// APIフレーム共通部分のサイズ(フレームデリミタ、データ長、チェックサム)
		DESTADDR64BIT_SIZE 							= 8,		// 64ビットアドレスサイズ
		DESTADDR16BIT_SIZE 							= 2,		// 16ビットアドレスサイズ
	};
	enum
	{
		// 送信 APIフレーム API ID
		AT_COMMAND 									= 0x08,
		AT_COMMAND_QUEUE_PARAMETER_VALUE 			= 0x09,
		TRANSMIT_REQUEST 							= 0x10,
		EXPLICIT_ADDRESSING_ZIGBEE_COMMAND		 	= 0x11,
		REMOTE_COMMAND_REQUEST 						= 0x17,
		CREATE_SOURCE_ROUTE 						= 0x21,
		// 受信 APIフレーム API ID
		AT_COMMAND_RESPONSE 						= 0x88,
		MODEM_STATUS 								= 0x8A,
		TRANSMIT_STATUS 							= 0x8B,
		RECEIVE_PACKET 								= 0x90,
		EXPLICIT_RX_INDICATOR						= 0x91,
		IO_DATA_SAMPLE_RX_INDICATOR		 			= 0x92,
		XBEE_SENSOR_READ_INDICATOR 					= 0x94,
		NODE_IDENTIFICATION_INDICATOR 				= 0x95,
		REMOTE_COMMAND_RESPONSE 					= 0x97,
		EXTENDED_MODEM_STATUS 						= 0x98,
		OVER_THE_AIR_FIRMWARE_UPDATE_STATUS 		= 0xA0,
		ROUTE_RECORD_INDICATOR 						= 0xA1,
		MANY_TO_ONE_ROUTE_REQUEST_INDICATOR 		= 0xA3
	};
	enum
	{
		FRAME_DELIMITER = 0x7E,									// フレームデリミタ
	};

    XBee( void );												// コンストラクタ
    XBee( HardwareSerial * serialr );							// コンストラクタ
    
	virtual ~XBee( void );										// デストラクタ
	
	virtual void init( void );									// 初期化処理
	virtual void init( HardwareSerial * serialr  );				// 初期化処理
	
	
	void func_hi( void );										// 高優先度処理：シリアル送受信処理
	void func_mid( void );										// 中優先度処理：受信データの解析
	void func_low( void );										// 低優先度処理：Reserved

	/* 送信フレーム用関数 */
	inline u1 TxATCommand(ATCommand *pframe, u1 len){ return transmit((XBeeTxFrame *)pframe, len ); };
	inline u1 TxATCommandQueu(ATCommandQueue *pframe, u1 len){ return transmit((XBeeTxFrame *)pframe, len ); };
	inline u1 TxTransmitRequest(TransmitRequest *pframe, u1 len){ return transmit((XBeeTxFrame *)pframe, len ); };
	inline u1 TxExplicitAddressing(ExplicitAddressing *pframe, u1 len){ return transmit((XBeeTxFrame *)pframe, len ); };
	inline u1 TxRemoteCommandRequest(RemoteCommandRequest *pframe, u1 len){ return transmit((XBeeTxFrame *)pframe, len ); };
	inline u1 TxCreateSourceRoute(CreateSourceRoute *pframe, u1 len){ return transmit((XBeeTxFrame *)pframe, len ); };

	/* 受信フレーム用関数 (派生先でオーバーライド) */
	inline virtual void RxATCommandResponse(ATCommand *pframe, u1 len ){}
	inline virtual void RxModemStatus( ModemStatus *pframe, u1 len){}
	inline virtual void RxTransmitStatus( TransmitStatus *pframe, u1 len){}
	inline virtual void RxReceivePacket( ReceivePacket *pframe, u1 len){}
	inline virtual void RxExplicitRxIndicator( ExplicitRxIndicator *pframe, u1 len){}
	inline virtual void RxIODataSampleRxIndicator( DataSampleRxIndicator *pframe, u1 len){}
	inline virtual void RxXBeeSensorReadIndicator( XBeeSensorReadIndicator *pframe, u1 len){}
	inline virtual void RxNodeIdentificationIndicator( NodeIdentificationIndicator *pframe, u1 len){}
	inline virtual void RxRemoteCommandResponse( RemoteCommandResponse *pframe, u1 len){}
	inline virtual void RxExtendedModemStatus( ExtendedModemStatus *pframe, u1 len){}
	inline virtual void RxOverTheAirFirmwareUpdateStatus( OverTheAirFirmwareUpdateStatus *pframe, u1 len){}
	inline virtual void RxRouteRecordIndicator( RouteRecordIndicator *pframe, u1 len){}
	inline virtual void RxManyToOneRouteRequestIndicator( ManyToOneRouteRequestIndicator *pframe, u1 len){}

private:
	void receive( void );										// フレーム受信処理
	u1 transmit( XBeeTxFrame *pkt, u1 len );					// フレーム送信処理

	RingBuffer 		m_TxBuff;									// 送信用リングバッファ
	RingBuffer 		m_RxBuff;									// 受信用リングバッファ
	XBeeAPIRxFrame	m_RxFrame;									// 受信フレーム(解析処理)
	u1				m_TxTemp[TXTEMP_SIZE];						// 送信用バッファ
	HardwareSerial *m_pSerial;									// シリアルオブジェクト
};

#endif
