
#include "stdtypes.h"
#include "RingBuffer.h"
#include "XBeeFrame.h"
#include "XBeeAPIRxFrame.h"
#ifdef DEBUG
#include "StubSerial.h"
#else
#include "HardwareSerial.h"
#endif

XBeeAPIRxFrame::XBeeAPIRxFrame( void )
{
	init();
};

XBeeAPIRxFrame::~XBeeAPIRxFrame( void )
{
};


void XBeeAPIRxFrame::init( void )
{
	m_sequence	= XBeeAPIRxFrame::RX_SEQ_IDLE;	// シーケンス：アイドル
	m_checksum	= ((u4)0U);
	m_length	= ((u1)0U);
	m_datacount	= ((u1)0U);
}

// 引数		: 受信バッファ
// 戻り値	: 成功(TRUE)/失敗(FALSE)
u1 XBeeAPIRxFrame::build( RingBuffer *pbuf )
{
	u1 ret;
	u1 cnt;
	u1 data;
	u1 data_en;
	u1 size;
	
	ret = FALSE;
	size = pbuf->size() + 1;								//チェックサムを考慮(+1)
	
#ifdef DEBUG
	Serial.print("Frame Build : ");
#endif
	for( cnt = 0; cnt < size; cnt++ )
	{
		data_en = pbuf->get(&data); 						//リングバッファからデータ取得
		
		if(TRUE == data_en)
		{
			set( data );									// フレームデータへデータ格納
			if(XBeeAPIRxFrame::RX_SEQ_END == m_sequence)	// フレーム構築完了？
			{
#ifdef DEBUG
			    Serial.print("\nBuild success!");
#endif
				ret = TRUE;
				cnt = size;
			}
			else if(XBeeAPIRxFrame::RX_SEQ_END <  m_sequence )	// 異常フレーム
			{
#ifdef DEBUG
			    Serial.print("\nBuild fail!");
#endif
				init();										// フレームデータ初期化
				cnt = size;
			}
			else
			{
			}
#ifdef DEBUG
		    Serial.print(data, 16);
		    Serial.print(" ");
#endif
		}
		else
		{
		}
	}
#ifdef DEBUG
	Serial.print("\n");
#endif
	return ret;
}

// 引数		: フレームデータ
void XBeeAPIRxFrame::set( u1 data )
{
	u1	sum;
	
	switch( m_sequence )
	{
	case XBeeAPIRxFrame::RX_SEQ_IDLE:							// 【アイドル状態】
		if( (u1)XBeeAPIRxFrame::FRAME_DELIMITER == data )		// フレームデリミタ確認
		{
			m_sequence = XBeeAPIRxFrame::RX_SEQ_LENGTH_H;		// データ長(MSB)へ
		}
		else
		{
		}
		break;
		
	case XBeeAPIRxFrame::RX_SEQ_LENGTH_H:						// 【データ長(MSB)状態】
		m_length = data;										// データ長格納
		m_sequence = XBeeAPIRxFrame::RX_SEQ_LENGTH_L;			// データ長(LSB)へ
		break;
		
	case XBeeAPIRxFrame::RX_SEQ_LENGTH_L:						// 【データ長(LSB)状態】
		m_length =( (0x00FF & m_length) << 8 ) | data;			// データ長格納
		if(1 < m_length)										// データ長確認
		{
			if( sizeof(XBeeRxFrame) > m_length)					// 受信バッファより大きい場合
			{
				m_datacount = ((u1)0U);							// フレームデータ長クリア
				m_sequence = XBeeAPIRxFrame::RX_SEQ_FRAME_DATA;	// データへ
			}
			else
			{
				m_sequence = XBeeAPIRxFrame::RX_SEQ_ERR_LENGTH;	// データ長異常
			}
		}
		else
		{
			m_sequence = XBeeAPIRxFrame::RX_SEQ_CHECK_SUM;		// チェックサムへ
		}
		break;
		
	case XBeeAPIRxFrame::RX_SEQ_FRAME_ESC:						// 【エスケープ処理状態】
		data = data ^ ESCAPE_XOR;								// データを0x20でXOR
		/* nobreak */
	case XBeeAPIRxFrame::RX_SEQ_FRAME_DATA:						// 【データ状態】
		if( XBeeAPIRxFrame::ESCAPE == data )					// エスケープ文字？
		{
			m_sequence = XBeeAPIRxFrame::RX_SEQ_FRAME_ESC;		// エスケープ処理へ
		}
		else
		{
			m_frame.buff[m_datacount] = data;					// フレームデータとしてデータを格納
			m_checksum += (u4)data;								// チェックサム計算
			m_datacount++;										// 格納したデータ長カウントアップ
			if(m_datacount >= m_length)							// データ長がフレームの終端に達したか？
			{
				m_sequence = XBeeAPIRxFrame::RX_SEQ_CHECK_SUM;	// チェックサム状態へ
			}
			else
			{
			}
		}
		break;
		
	case XBeeAPIRxFrame::RX_SEQ_CHECK_SUM:						// 【チェックサム】
		sum = (u1)(((u1)XBeeAPIRxFrame::CHECKSUM_CONST) - (0x000000FF & m_checksum));	// チェックサム
		if( data == sum )
		{
			m_sequence = XBeeAPIRxFrame::RX_SEQ_END;		// フレーム構築完了
		}
		else
		{
			m_sequence = XBeeAPIRxFrame::RX_SEQ_ERR_SUM;	// チェックサム異常
		}
		break;
		
	default:
		break;
	}
}

