#ifndef __XBEEAPIFRAME_H__
#define __XBEEAPIFRAME_H__

#define XBEEFRAME_MAX_SIZE	(128)

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
} Common;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 AT_command[2];
	u1 Command_status;
	u1 Command_data;
} ATCommandResponse;

typedef struct
{
	u1 Frame_type;
	u1 Status;
} ModemStatus;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 DestAddr_16_h;
	u1 DestAddr_16_l;
	u1 trasmit_retry;
	u1 delivery_status;
	u1 discovery_status;
} TransmitStatus;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 DestAddr_64_1;
	u1 DestAddr_64_2;
	u1 DestAddr_64_3;
	u1 DestAddr_64_4;
	u1 DestAddr_64_5;
	u1 DestAddr_64_6;
	u1 DestAddr_64_7;
	u1 DestAddr_64_8;
	u1 DestAddr_16_h;
	u1 DestAddr_16_l;
	u1 Receive_options;
	u1 Receive_data[0];	//可変長
} ReceivePacket;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 DestAddr_64_1;
	u1 DestAddr_64_2;
	u1 DestAddr_64_3;
	u1 DestAddr_64_4;
	u1 DestAddr_64_5;
	u1 DestAddr_64_6;
	u1 DestAddr_64_7;
	u1 DestAddr_64_8;
	u1 DestAddr_16_h;
	u1 DestAddr_16_l;
	u1 Source_endpoint;
	u1 Destination_endpoint;
	u1 Cluster_ID;
	u1 Profile_ID;
	u1 Receive_options;
	u1 Receive_data[0];
} ExplicitRxIndicator;

typedef struct
{
	u1 Frame_type;
} DataSampleRxIndicator;

typedef struct
{
	u1 Frame_type;
} XBeeSensorReadIndicator;

typedef struct
{
	u1 Frame_type;
} NodeIdentificationIndicator;

typedef struct
{
	u1 Frame_type;
} RemoteCommandResponse;

typedef struct
{
	u1 Frame_type;
} ExtendedModemStatus;

typedef struct
{
	u1 Frame_type;
} OverTheAirFirmwareUpdateStatus;

typedef struct
{
	u1 Frame_type;
} RouteRecordIndicator;

typedef struct
{
	u1 Frame_type;
} ManyToOneRouteRequestIndicator;

typedef union
{
	u1 								buff[XBEEFRAME_MAX_SIZE];
	Common							com;
	ATCommandResponse 				atcr;
	ModemStatus 					ms;
	TransmitStatus 					ts;
	ReceivePacket 					rp;
	ExplicitRxIndicator 			eri;
	DataSampleRxIndicator 			dsr;
	XBeeSensorReadIndicator 		xbsri;
	NodeIdentificationIndicator 	nii;
	RemoteCommandResponse 			rcr;
	ExtendedModemStatus 			ems;
	OverTheAirFirmwareUpdateStatus 	otafus;
	RouteRecordIndicator 			rri;
	ManyToOneRouteRequestIndicator 	mtorri;
} XBeeRxFrame;


typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 AT[2];
//	u1 Param[0];		// 可変長
} ATCommand;

typedef ATCommand ATCommandQueue;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 DestAddr_64_1;
	u1 DestAddr_64_2;
	u1 DestAddr_64_3;
	u1 DestAddr_64_4;
	u1 DestAddr_64_5;
	u1 DestAddr_64_6;
	u1 DestAddr_64_7;
	u1 DestAddr_64_8;
	u1 DestAddr_16_h;
	u1 DestAddr_16_l;
	u1 BroadcastRadius;
	u1 Options;
//	u1 Data[64];			//可変長
} TransmitRequest;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 DestAddr_64_1;
	u1 DestAddr_64_2;
	u1 DestAddr_64_3;
	u1 DestAddr_64_4;
	u1 DestAddr_64_5;
	u1 DestAddr_64_6;
	u1 DestAddr_64_7;
	u1 DestAddr_64_8;
	u1 DestAddr_16_h;
	u1 DestAddr_16_l;
	u1 SrcEndPoint;
	u1 DestEndPoint;
	u1 ClusterID_h;
	u1 ClusterID_l;
	u1 ProfileID_h;
	u1 ProfileID_l;
	u1 BroadcastRadius;
	u1 TransmissionOptions;
//	u1 Data[64];			//可変長
} ExplicitAddressing;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 DestAddr_64_1;
	u1 DestAddr_64_2;
	u1 DestAddr_64_3;
	u1 DestAddr_64_4;
	u1 DestAddr_64_5;
	u1 DestAddr_64_6;
	u1 DestAddr_64_7;
	u1 DestAddr_64_8;
	u1 DestAddr_16_h;
	u1 DestAddr_16_l;
	u1 RemoteCommandOptions;
	u1 AT[2];
//	u1 Command[64];		//可変長
} RemoteCommandRequest;

typedef struct
{
	u1 Frame_type;
	u1 Frame_ID;
	u1 DestAddr_64_1;
	u1 DestAddr_64_2;
	u1 DestAddr_64_3;
	u1 DestAddr_64_4;
	u1 DestAddr_64_5;
	u1 DestAddr_64_6;
	u1 DestAddr_64_7;
	u1 DestAddr_64_8;
	u1 DestAddr_16_h;
	u1 DestAddr_16_l;
	u1 RouteCommandOptions;
	u1 NumOfAddress;
	u1 Address1_h;
	u1 Address1_l;
	u1 Address2_h;
	u1 Address2_l;
	u1 Address3_h;
	u1 Address3_l;		//可変長
} CreateSourceRoute;

typedef union
{
	u1 								buff[XBEEFRAME_MAX_SIZE];
	Common							com;
	ATCommand						atc;
	ATCommandQueue					atcq;
	TransmitRequest					tq;
	ExplicitAddressing				ea;
	RemoteCommandRequest			rcr;
	CreateSourceRoute				csr;
} XBeeTxFrame;

#endif
